
package org.mule.modules.quickbooks.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.quickbooks.config.CreateSalesTermMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.w3c.dom.Element;

public class CreateSalesTermDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public CreateSalesTermDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(CreateSalesTermMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("accessToken")!= null)&&(!StringUtils.isBlank(element.getAttribute("accessToken")))) {
            builder.addPropertyValue("accessToken", element.getAttribute("accessToken"));
        }
        if ((element.getAttribute("accessTokenSecret")!= null)&&(!StringUtils.isBlank(element.getAttribute("accessTokenSecret")))) {
            builder.addPropertyValue("accessTokenSecret", element.getAttribute("accessTokenSecret"));
        }
        if ((element.getAttribute("name")!= null)&&(!StringUtils.isBlank(element.getAttribute("name")))) {
            builder.addPropertyValue("name", element.getAttribute("name"));
        }
        if ((element.getAttribute("dueDays")!= null)&&(!StringUtils.isBlank(element.getAttribute("dueDays")))) {
            builder.addPropertyValue("dueDays", element.getAttribute("dueDays"));
        }
        if ((element.getAttribute("discountDays")!= null)&&(!StringUtils.isBlank(element.getAttribute("discountDays")))) {
            builder.addPropertyValue("discountDays", element.getAttribute("discountDays"));
        }
        if ((element.getAttribute("discountPercent")!= null)&&(!StringUtils.isBlank(element.getAttribute("discountPercent")))) {
            builder.addPropertyValue("discountPercent", element.getAttribute("discountPercent"));
        }
        if ((element.getAttribute("dayOfMonthDue")!= null)&&(!StringUtils.isBlank(element.getAttribute("dayOfMonthDue")))) {
            builder.addPropertyValue("dayOfMonthDue", element.getAttribute("dayOfMonthDue"));
        }
        if ((element.getAttribute("dueNextMonthDays")!= null)&&(!StringUtils.isBlank(element.getAttribute("dueNextMonthDays")))) {
            builder.addPropertyValue("dueNextMonthDays", element.getAttribute("dueNextMonthDays"));
        }
        if ((element.getAttribute("discountDayOfMonth")!= null)&&(!StringUtils.isBlank(element.getAttribute("discountDayOfMonth")))) {
            builder.addPropertyValue("discountDayOfMonth", element.getAttribute("discountDayOfMonth"));
        }
        if ((element.getAttribute("dateDiscountPercent")!= null)&&(!StringUtils.isBlank(element.getAttribute("dateDiscountPercent")))) {
            builder.addPropertyValue("dateDiscountPercent", element.getAttribute("dateDiscountPercent"));
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
            if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                propertyValues.addPropertyValue("messageProcessors", new ManagedList());
            }
            List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
            listMessageProcessors.add(definition);
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
